# Generated by Django 3.0.3 on 2021-03-30 02:47

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='IPAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.GenericIPAddressField(unique=True, verbose_name='IP Address')),
                ('init_visit', models.DateTimeField(null=True, verbose_name='Initial visit')),
                ('last_visit', models.DateTimeField(null=True, verbose_name='Most recent visit')),
            ],
        ),
    ]



from django.utils import timezone

from .models import IPAddress
from .utils import get_ip_address

class LogIPMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        ip = get_ip_address(request)
        # log time now to prevent init and last visit
        # from being different even by a second
        now = timezone.now()
        ipaddr, _ = IPAddress.objects.get_or_create(
            ip=ip,
        )
        ipaddr.last_visit = now
        if ipaddr.init_visit is None:
            ipaddr.init_visit = now
        ipaddr.save()
        response = self.get_response(request)
        return response



def get_ip_address(request):
    presumed_ip = request.META.get('HTTP_X_FORWARDED_FOR')
    return \
        presumed_ip.split(',')[0] \
            if presumed_ip is not None \
                else \
                    request.META.get('REMOTE_ADDR')




from time import sleep
from django.test import TestCase
from django.urls import reverse
from django.utils import timezone

from .models import IPAddress
from .utils import get_ip_address


class MiddlewareTestCase(TestCase):
    def ping(self):
        url = reverse('ipaddr:index')
        return self.client.get(url)

    def test_new_ip_will_be_saved(self):
        response = self.ping()
        request = response.wsgi_request
        ip = get_ip_address(request)
        newly_saved_ip = IPAddress.objects.get(ip=ip)
        self.assertEqual(newly_saved_ip.init_visit, newly_saved_ip.last_visit)

    def test_already_logged_ip_will_change_last_visit(self):
        self.ping()
        # simulate another visit by same address
        sleep(2)
        response = self.ping()
        request = response.wsgi_request
        ip = get_ip_address(request)
        logged_ip = IPAddress.objects.get(ip=ip)
        self.assertNotEqual(logged_ip.init_visit, logged_ip.last_visit)



from django.db import models


class IPAddress(models.Model):
    ip = models.GenericIPAddressField(unique=True, verbose_name="IP Address")
    init_visit = models.DateTimeField(null=True, verbose_name="Initial visit")
    last_visit = models.DateTimeField(null=True, verbose_name="Most recent visit")

    def __str__(self):
        return self.ip

    class Meta:
        verbose_name = "IP address"
        verbose_name_plural = "IP addresses"

